@@ -53,6 +53,9 @@
 #include <math.h>
 #include "defs.h" // defines _cerf_cmplx, NaN, C, cexp, ...
 
+// for analysing the algorithm:
+IMPORT extern int faddeeva_algorithm;
+
 const double spi2 = 0.8862269254527580136490837416705725913990; // sqrt(pi)/2
 const double s2pi = 2.5066282746310005024157652848110; // sqrt(2*pi)
 const double pi   = 3.141592653589793238462643383279503;
@@ -67,6 +70,11 @@ _cerf_cmplx cerfcx(_cerf_cmplx z)
     // the complex underflow-compensated complementary error function,
     // trivially related to Faddeeva's w_of_z.
 
+#if defined(__clang_major__)
+    // Work around clang bug: creal(0+I*inf)=>NaN
+    if (cimag(z)==0)
+        return erfcx(creal(z));
+#endif
     return w_of_z(C(-cimag(z), creal(z)));
 }
 
@@ -134,23 +142,22 @@ double voigt( double x, double sigma, double gamma )
     if ( gam==0 ) {
         if ( sig==0 ) {
             // It's kind of a delta function
+            faddeeva_algorithm = 801;
             return x ? 0 : Inf;
         } else {
-            // It's a pure Gaussian
-            return exp( -x*x/2/(sig*sig) ) / s2pi / sig;
+            // It's a pure Gaussian (optimized, only 1 run-time division)
+            faddeeva_algorithm = 802;
+            return exp( -0.5*(x*(1/sig))*(x*(1/sig)) ) * (1/s2pi) * (1/sig);
         }
     } else {
         if ( sig==0 ) {
             // It's a pure Lorentzian
-            return gam / pi / (x*x + gam*gam);
+            faddeeva_algorithm = 803;
+            return gam / (pi * (x*x + gam*gam));
         } else {
             // Regular case, both parameters are nonzero
-            _cerf_cmplx z = C(x,gam) / sqrt(2) / sig;
-            return creal( w_of_z(z) ) / s2pi / sig;
-            // TODO: correct and activate the following:
-//            double w = sqrt(gam*gam+sig*sig); // to work in reduced units
-//            _cerf_cmplx z = C(x/w,gam/w) / sqrt(2) / (sig/w);
-//            return creal( w_of_z(z) ) / s2pi / (sig/w);
+            _cerf_cmplx z = C(x,gam) * sqrt(.5) * (1/sig);
+            return creal( w_of_z(z) ) * (1/s2pi) * (1/sig);
         }
     }
 }
